/* Copyright 2019 Henning Marxen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include QMK_KEYBOARD_H
#define HIGHEST_LAYER 4 //replace the number with your number of layers
static uint8_t current_layer = 1;

enum l01_layers {
	_TOP,
	_F,
	_BACKLIGHT,
	_RGB
};

enum l01_keycodes {
    KC_LUP = SAFE_RANGE, //cycle layers in up direction
    KC_LDN, //cycle layers in down direction
	DS_TOGG //Toggle on off the 7-segment display
};

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

/* This layout is meant to work for both Ansi and ISO users out of the box. The unused keys in your respective layout will stay unconnected and do no harm.
 * If you use the split space keys or a split backspace key you might want to flip Right Alt and the Right OS key or flip Home and Delete.
 *
 * ,------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------.
 * | Esc        | 1         | 2         | 3         | 4         | 5         | 6         | 7         | 8         | 9         | 0         | -         | =         | Backspace | Home      | ~         |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * | Tab              | Q         | W         | E         | R         | T         | Y         | U         | I         | O         | P         | [         | ]         | \               | Delete    |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * | Caps                | A         | S         | D         | F         | G         | H         | J         | K         | L         | ;         | '         | #         |        Return| Layer +   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * | Shift         | \         | Z         | X         | C         | V         | B         | N         | M         | ,         | .         | /         | Shift              | Up        | Layer -   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * | Ctrl          | OS            | Alt           |           Space         |             Space           | OS            | Alt           | Ctrl          |    | Left      | Down      | Right     |
 * `-------------------------------------------------------------------------------------------------------------------------------------------------------'    `-----------------------------------'
 */	
	[_TOP] = LAYOUT(
		KC_ESC,		KC_1,		MO(_F),		MO(_BACKLIGHT),		MO(_RGB),		KC_5,		KC_6,		KC_7,		KC_8,		KC_9,		KC_0,		KC_MINS,	KC_EQL,		KC_BSPC,	KC_HOME,	KC_GRV,
		KC_TAB,		KC_Q,		KC_W,		KC_E,		KC_R,		KC_T,		KC_Y,		KC_U,		KC_I,		KC_O,		KC_P,		KC_LBRC,	KC_RBRC,	KC_BSLS,	KC_DEL,
		KC_CAPS,	KC_A,		KC_S,		KC_D,		KC_F,		KC_G,		KC_H,		KC_J,		KC_K,		KC_L,		KC_SCLN,	KC_QUOT,	KC_NUHS,	KC_ENT,		KC_LUP,
		KC_LSFT,	KC_NUBS,	KC_Y,		KC_X,		KC_C,		KC_V,		KC_B,		KC_N,		KC_M,		KC_COMM,	KC_DOT,		KC_SLSH,	KC_RSFT,	KC_UP,		KC_LDN,
		KC_LCTL,	KC_LWIN,	KC_LALT,							KC_SPC,					KC_SPC,					KC_RWIN,	KC_RALT,	KC_RCTL,				KC_LEFT,	KC_DOWN,	KC_RIGHT		
	),

/* 
 * ,------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------.
 * |            | F1        | F2        | F3        | F4        | F5        | F6        | F7        | F8        | F9        | F10       | F11       | F12       |           |           |           |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |                  | F13       | F14       | F15       | F16       | F17       | F18       | F19       | F20       | F21       | F22       | F23       | F24       |                 | Mute      |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |                     |           |           |           |           |           |           |           |           |           |           |           |           |              | Layer +   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |               |           |           |           |           |           |           |           |           |           |           |           |                    | Volume+   | Layer -   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |               |               |               |                         |                             |               |               |               |    |Brightness-| Volume-   |Brightness+|
 * `-------------------------------------------------------------------------------------------------------------------------------------------------------'    `-----------------------------------'
 */
	[_F] = LAYOUT(
		XXXXXXX,	KC_F1,		KC_F2,		KC_F3,		KC_F4,		KC_F5,		KC_F6,		KC_F7,		KC_F8,		KC_F9,		KC_F10,		KC_F11,		KC_F12,		XXXXXXX,	XXXXXXX,	XXXXXXX,
		XXXXXXX,	KC_F13,		KC_F14,		KC_F15,		KC_F16,		KC_F17,		KC_F18,		KC_F19,		KC_F20,		KC_F21,		KC_F22,		KC_F23,		KC_F24,		XXXXXXX,	KC_MUTE,
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	KC_LUP,
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	KC_VOLU,	KC_LDN,	
		XXXXXXX,	XXXXXXX,	XXXXXXX,							XXXXXXX,				XXXXXXX,				XXXXXXX,	XXXXXXX,	XXXXXXX,				KC_BRID,	KC_VOLD,	KC_BRIU		
	),

/* 
 * ,------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------.
 * |            |           |           |           |           |           |           |           |           |           |           |           |           |           |           |           |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |                  |           |           |           |           |           |           |           |           |           |           |           |           |                 |           |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |                     |           |           |           |           |           |           |           |           |           |           |           |           |              | Layer +   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |               |           |           |           |           |           |           |           |           |           |           |           | BL Toggle          | BL+       | Layer -   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |               |               |               |                         |                             |               |               |               |    | BL Cylcle | BL-       | BL Breathe|
 * `-------------------------------------------------------------------------------------------------------------------------------------------------------'    `-----------------------------------'
 */
	[_BACKLIGHT] = LAYOUT(
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	DS_TOGG,	KC_LUP,
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	BL_TOGG,	BL_INC,		KC_LDN,
		XXXXXXX,	XXXXXXX,	XXXXXXX,							XXXXXXX,				XXXXXXX,				XXXXXXX,	XXXXXXX,	XXXXXXX,				BL_STEP,	BL_DEC,		BL_BRTG		
	),
	
/* 
 * ,------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------.
 * |            |           |           |           |           |           |           |           |           |           |           |           |           |           |           |           |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |                  |           |           |           | Rainbow   | Test Mode |           |           |           |           | Plain Mode|           |           |                 |           |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |                     |           | Swirl Mode|           |           | Gradient  |           |           | Knight    |           |           |           |           | RGB Toggle   | Layer +   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |               |           |           | Xmas Mode |           |           | Breathing |           |           |           |Saturation-|Saturation+|                    |Brightness+| Layer -   |
 * |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
 * |               |               |               |                         |                             |               | Hue-          | Hue+          |    | RGB Mode- |Brightness-| RGB Mode+ |
 * `-------------------------------------------------------------------------------------------------------------------------------------------------------'    `-----------------------------------'
 */
	[_RGB] = LAYOUT(
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,
		XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	RGB_M_R,	RGB_M_T,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	RGB_M_P,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,
		XXXXXXX,	XXXXXXX,	RGB_M_SW,	XXXXXXX,	XXXXXXX,	RGB_M_G,	XXXXXXX,	XXXXXXX,	RGB_M_K,	XXXXXXX,	XXXXXXX,	XXXXXXX,	XXXXXXX,	RGB_TOG,	KC_LUP,
		XXXXXXX,	XXXXXXX,	XXXXXXX,	RGB_M_X,	XXXXXXX,	XXXXXXX,	RGB_M_B,	XXXXXXX,	XXXXXXX,	XXXXXXX,	RGB_SAD,	RGB_SAI,	XXXXXXX,	RGB_VAI,	KC_LDN,
		XXXXXXX,	XXXXXXX,	XXXXXXX,							XXXXXXX,				XXXXXXX,				XXXXXXX,	RGB_HUD,	RGB_HUI,				RGB_RMOD,	RGB_VAD,	RGB_MOD		
	),
};

bool process_record_user(uint16_t keycode, keyrecord_t *record) {
	switch (keycode) {
		case KC_LUP:
			if(record->event.pressed) {
			} else {
				if (current_layer == HIGHEST_LAYER){
					for (int i=1; i<=HIGHEST_LAYER; i++){
						layer_off(i);
					}
					current_layer=1;
				} else { 
					current_layer++;
					layer_on(current_layer);
				}				
			}
			return false;
		case KC_LDN:
			if(record->event.pressed) {
			} else {
				if (current_layer == 1){
					for (int i=1; i<=HIGHEST_LAYER; i++){
						layer_on(i);
					}
					current_layer=HIGHEST_LAYER;
				} else {
					layer_off(current_layer);
					current_layer--;
				}				
			}
			return false;
		case DS_TOGG:
			if(record->event.pressed) {

			}
			return false;
		default:
			return true;
	}
}    

