/* Copyright 2019 Henning Marxen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "l01.h"
uint8_t layer = 0;

void keyboard_pre_init_kb(void) {
	setPinOutput(B7); //Caps LED
	setPinOutput(E6); //Clock
	setPinOutput(F1); //Data
	setPinOutput(F0); //Latch
	setPinOutput(C6); //Disp. PWM
	writePinLow(B7);
	writePinLow(E6);
	writePinLow(F1);
	writePinLow(F0);
	writePinLow(C6);
}

void delay(uint16_t milliSeconds) 
{  
	uint32_t start_time = timer_read(); 
	while (timer_read() < start_time + milliSeconds); 
}

//   __8___
// 3|     |7
//  |__2__|
// 4|     |6
//  |_____|  *1
//     5
//
// The 7-segment display is run by a TLC6C598PWR shift register and LED driver. The shift register takes an 8-bit input and
// turns the LEDs on or off by the bit value. The least significant bit switches the decimal point and the most significant
// the top most horizontal line, as seen in the diagram above. E.g. to display "2" the binary value would be 11011010 which
// equals 218 decimal. To send a "0" bit set pin E6 high. To send a "1" bit set pin F1 and then E6 high. Pin F0 latches the
// last 8 bits into the memory. The TLC6C598PWR can receive data only up to 6 MHz, so a bit of a delay is needed after each
// input.
//                                   0    1    2    3    4    5    6    7    8    9    A    b    C    d    E    F   |--Start-Animation---|
static uint8_t dispNumbers[21] = { 252,  96, 218, 242, 102, 182, 190, 224, 254, 246, 238,  62, 156, 122, 158, 142,  48,  24,  12, 132, 192 };

// This function takes the dispNumbers array and turns it's contend into binary and sends it to the TLC6C598PWR.
void display_refresh(bool decPoint, uint8_t number) {
	uint8_t layerChar = dispNumbers[number];
	if (decPoint == true) {
		layerChar += 1;
	}
	for (uint8_t i = 0; i <= 7; i++) {
		if ((layerChar & 1) == 1){
			writePinHigh(F1);
			delay(1);
			writePinHigh(E6);
			delay(1);
			writePinLow(F1);
		} else {
			writePinHigh(E6);
		}
		layerChar = layerChar >> 1;
		delay(1);
		writePinLow(E6);
		delay(1);
	}
	writePinHigh(F0);
	delay(1);
	writePinLow(F0);
	delay(1);
}

void keyboard_post_init_kb(void) {
	bool caps = false;
	if (host_keyboard_leds() & (1<<USB_LED_CAPS_LOCK)){
		caps = true;
	}
	//This is the 7-segment display boot animation
	display_refresh(caps, 1);
	delay(100);
	display_refresh(caps, 16);
	delay(100);
	display_refresh(caps, 17);
	delay(100);
	display_refresh(caps, 18);
	delay(100);
	display_refresh(caps, 19);
	delay(100);
	display_refresh(caps, 20);
	delay(100);
	display_refresh(caps, 1);
	delay(100);
	display_refresh(caps, 16);
	delay(100);
	display_refresh(caps, 17);
	delay(100);
	display_refresh(caps, 18);
	delay(100);
	display_refresh(caps, 19);
	delay(100);
	display_refresh(caps, 20);
	delay(100);
	display_refresh(caps, 1);
}

//Make the CAPSLOCK LED check if CAPS is locked
bool led_update_kb(led_t led_state) {
    bool res = led_update_user(led_state);
    if(res) {
		//If JP1 and JP2 are briged the backlight of the Caps Lock key is driven by pin B7
        writePin(B7, led_state.caps_lock);
		//The 7-segment display decimal point is used as a Caps Lock LED
		display_refresh(led_state.caps_lock, layer + 1);
    }
    return res;
}

void matrix_init_kb(void) {
	matrix_init_user();
}

void matrix_scan_kb(void) {
	matrix_scan_user();
}

uint32_t layer_state_set_kb(uint32_t state) {
	//This updates the display on every layer change
	layer = biton32(state);
	if (host_keyboard_leds() & (1<<USB_LED_CAPS_LOCK)){
		display_refresh(true, layer + 1);
	} else {
		display_refresh(false, layer + 1);
	}
	return state;
}