/* Copyright 2019 Henning Marxen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef L01_H
#define L01_H

#include "quantum.h"
#define XXX KC_NO

/* L01 matrix layout
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|Back|Back| Del|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]|      \| PUp|
 * |-------------------------------------------------------------------------------|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|   #|Return| PDo|
 * |-------------------------------------------------------------------------------|
 * |Shift|   \|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|   Space   |    Space    | Alt|   OS| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
// The first section contains all of the arguments
// The second converts the arguments into a two-dimensional array
#define LAYOUT( \
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53, k63, k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55, k62, k65, \
	k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54, k57, k64, k67, \
	k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
	k40, k41, k42,      k44,      k46,      k48, k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
    { k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
    { k40, k41, k42, XXX, k44, XXX, k46, XXX, k48, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, k57, k58, k59, k5A }, \
	{ XXX, k61, k62, k63, k64, k65, k66, k67, k68, k69, k6A }  \
}
#define KEYMAP LAYOUT

/* Ansi
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|   BackSp| Del|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]|      \| PUp|
 * |-------------------------------------------------------------------------------|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|     Return| PDo|
 * |-------------------------------------------------------------------------------|
 * |     Shift|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|            Space             |  Alt| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_ansi(  \
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53,      k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55, k62, k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54,      k64, k67, \
	k30,      k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,                k46,           k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, XXX, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, XXX, XXX, XXX, k46, XXX, XXX, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, XXX, k58, k59, k5A }, \
	{ XXX, k61, k62, XXX, k64, k65, k66, k67, k68, k69, k6A }  \
}

/* Ansi_SplitBackspace
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|Back| Del|Home|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]|      \| PUp|
 * |-------------------------------------------------------------------------------|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|     Return| PDo|
 * |-------------------------------------------------------------------------------|
 * |     Shift|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|            Space             |  Alt| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_ansi_splitbackspace(\
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53, k63, k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55, k62, k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54,      k64, k67, \
	k30,      k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,                k46,           k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, XXX, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, XXX, XXX, XXX, k46, XXX, XXX, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, XXX, k58, k59, k5A }, \
	{ XXX, k61, k62, k63, k64, k65, k66, k67, k68, k69, k6A }  \
}

/* Ansi_SplitSpace
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|   BackSp| Del|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]|      \| PUp|
 * |-------------------------------------------------------------------------------|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|     Return| PDo|
 * |-------------------------------------------------------------------------------|
 * |     Shift|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|   Space   |    Space    | Alt|   OS| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_ansi_splitspace(\
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53,      k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55, k62, k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54,      k64, k67, \
	k30,      k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,      k44,      k46,      k48, k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, XXX, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, k43, XXX, k45, k46, k47, XXX, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, XXX, k58, k59, k5A }, \
	{ XXX, k61, k62, XXX, k64, k65, k66, k67, k68, k69, k6A }  \
}

/* Ansi_SplitBackspace_SplitSpace
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|Back| Del|Home|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]|      \| PUp|
 * |-------------------------------------------------------------------------------|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|     Return| PDo|
 * |-------------------------------------------------------------------------------|
 * |     Shift|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|   Space   |    Space    | Alt|   OS| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_ansi_splitbackspace_splitspace(\
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53, k63, k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55, k62, k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54,      k64, k67, \
	k30,      k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,      k44,      k46,      k48, k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, XXX, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, k43, XXX, k45, k46, k47, XXX, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, XXX, k58, k59, k5A }, \
	{ XXX, k61, k62, k63, k64, k65, k66, k67, k68, k69, k6A }  \
}

/* ISO
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|   BackSp| Del|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]| Return| PUp|
 * |--------------------------------------------------------------------|     |----|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|    #|     | PDo|
 * |-------------------------------------------------------------------------------|
 * |Shift|   \|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|            Space             |  Alt| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_iso(\
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53,      k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55,      k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54, k57, k64, k67, \
	k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,                k46,           k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, XXX, XXX, XXX, k46, XXX, XXX, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, k57, k58, k59, k5A }, \
	{ XXX, k61, XXX, XXX, k64, k65, k66, k67, k68, k69, k6A }  \
}

/* ISO_SplitBackspace
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|Back| Del|Home|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]| Return| PUp|
 * |--------------------------------------------------------------------|     |----|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|    #|     | PDo|
 * |-------------------------------------------------------------------------------|
 * |Shift|   \|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|            Space             |  Alt| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_iso_splitbackspace(\
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53, k63, k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55,      k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54, k57, k64, k67, \
	k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,                k46,           k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, XXX, XXX, XXX, k46, XXX, XXX, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, k57, k58, k59, k5A }, \
	{ XXX, k61, XXX, k63, k64, k65, k66, k67, k68, k69, k6A }  \
}

/* ISO_SplitSpace
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|   BackSp| Del|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]| Return| PUp|
 * |--------------------------------------------------------------------|     |----|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|    #|     | PDo|
 * |-------------------------------------------------------------------------------|
 * |Shift|   \|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|   Space   |    Space    | Alt|   OS| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_iso_splitspace(\
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53,      k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55,      k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54, k57, k64, k67, \
	k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,      k44,      k46,      k48, k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, XXX, k44, XXX, k46, XXX, k48, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, k57, k58, k59, k5A }, \
	{ XXX, k61, XXX, XXX, k64, k65, k66, k67, k68, k69, k6A }  \
}

/* ISO_SplitBackspace_SplitSpace
 * ,-------------------------------------------------------------------------------.
 * | Esc|   1|   2|   3|   4|   5|   6|   7|   8|   9|   0|   -|   =|Back| Del|Home|
 * |-------------------------------------------------------------------------------|
 * |   Tab|   Q|   W|   E|   R|   T|   Y|   U|   I|   O|   P|   [|   ]| Return| PUp|
 * |--------------------------------------------------------------------|     |----|
 * |   Caps|   A|   S|   D|   F|   G|   H|   J|   K|   L|   ;|   '|    #|     | PDo|
 * |-------------------------------------------------------------------------------|
 * |Shift|   \|   Z|   X|   C|   V|   B|   N|   M|   ,|   .|   /|   Shift|  Up| End|
 * |-------------------------------------------------------------------------------|
 * | Ctrl|   OS|  Alt|   Space   |    Space    | Alt|   OS| Ctrl|   |Left|Down|Righ|
 * `------------------------------------------------------------'   `--------------'
 */
#define LAYOUT_iso_splitbackspace_splitspace(\
    k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A, k56, k51, k53, k63, k66, \
	k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A, k52, k55,      k65, \
    k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A, k54, k57, k64, k67, \
	k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A, k5A, k59, k6A, k69, \
    k40, k41, k42,      k44,      k46,      k48, k49, k4A, k58, k61, k68 \
) \
{ \
    { k00, k01, k02, k03, k04, k05, k06, k07, k08, k09, k0A }, \
	{ k10, k11, k12, k13, k14, k15, k16, k17, k18, k19, k1A }, \
	{ k20, k21, k22, k23, k24, k25, k26, k27, k28, k29, k2A }, \
	{ k30, k31, k32, k33, k34, k35, k36, k37, k38, k39, k3A }, \
	{ k40, k41, k42, XXX, k44, XXX, k46, XXX, k48, k49, k4A }, \
	{ XXX, k51, k52, k53, k54, k55, k56, k57, k58, k59, k5A }, \
	{ XXX, k61, XXX, k63, k64, k65, k66, k67, k68, k69, k6A }  \
}

#endif